@@ -240,7 +240,7 @@ woebin2_tree = function(dtm, initial_binning, min_perc_total=0.02, stop_limit=0.
 # examples
 # system.time( initial_binning <- woebin2_init_bin(dtm, min_perc_total) )
 # system.time( woebin2_tree_1bst(dtm, initial_binning, min_perc_total) )
-# system.time(woebin2_tree(dtm, initial_binning, min_perc_total))
+# system.time( woebin2_tree(dtm, initial_binning, min_perc_total) )
 
 # required in woebin2 # # format binning output
 binning_format = function(binning) {
@@ -305,7 +305,7 @@ woebin2 = function(dt, y, x, breaks=NULL, min_perc_total=0.02, stop_limit=0.1, m
 
 #' WOE Binning
 #'
-#' \code{woebin} generates optimal binning for both numerical and categorical variables using tree-like segmentation. For the categorical variables, the binning segmentation will ordered by the levels for factor and by the bad probability for character. \code{woebin} can also customizing breakpoints for both numerical and categorical variables.
+#' \code{woebin} generates optimal binning using tree-like segmentation for numerical, factor and categorical variables. \code{woebin} can also customizing breakpoints if the breaks_list was provided.
 #'
 #' @name woebin
 #' @param dt A data frame with both x (predictor/feature) and y (response/label) variables.
@@ -361,6 +361,8 @@ woebin2 = function(dt, y, x, breaks=NULL, min_perc_total=0.02, stop_limit=0.1, m
 #' @export
 #'
 woebin = function(dt, y, x=NULL, breaks_list=NULL, min_perc_total=0.02, stop_limit=0.1, max_bin_num=5, positive="bad|1", print_step=1L) {
+  # global variables
+  # detectCores = makeCluster = parLapply = NULL
 
   # set dt as data.table
   dt = setDT(dt)
@@ -425,7 +427,6 @@ woebin = function(dt, y, x=NULL, breaks_list=NULL, min_perc_total=0.02, stop_lim
 
 
 
-
   # binning for each x variable ------
   x_num = 1
   xs_len = length(xs)
@@ -461,6 +462,9 @@ woebin = function(dt, y, x=NULL, breaks_list=NULL, min_perc_total=0.02, stop_lim
     )
   }
 
+
+  # bins = germancredit[, sapply(.SD, woebin2, y, breaks_list, min_perc_total, stop_limit, max_bin_num), .SDcols = xs]
+
   return(bins)
 }
 
@@ -710,6 +714,8 @@ woebin_plot = function(bins, x=NULL, title=NULL) {
 #' @param bins A list or data frame. Binning information generated by \code{woebin}.
 #' @param dt A data frame.
 #' @param y Name of y variable.
+#' @param all_var Logical, default FALSE. It means whether to adjust all variables' binning breaks.
+#' @param count_distr_limit The count_distr limit to adjust binning breaks, default 0.05.
 #'
 #' @seealso  \code{\link{woebin}}, \code{\link{woebin_ply}}, \code{\link{woebin_plot}}
 #'
@@ -737,9 +743,9 @@ woebin_plot = function(bins, x=NULL, title=NULL) {
 #' @importFrom graphics hist plot
 #' @export
 #'
-woebin_adj = function(bins, dt, y) {
+woebin_adj = function(bins, dt, y, all_var = FALSE, count_distr_limit = 0.05) {
   # global variables or functions
-  variable = p = bin_adj = NULL
+  . = V1 = badprob = badprob2 = bin2 = bin_adj = count_distr = variable = x_breaks = x_class = NULL
 
   dt = setDT(dt)
   # bins # if (is.list(bins)) rbindlist(bins)
@@ -752,9 +758,11 @@ woebin_adj = function(bins, dt, y) {
   }
 
   # function woebin adj plotting
-  show_binadj_plot = function(dt, y, x, breaks) {
-    eval(parse(text = paste0(
-      "bin_adj=woebin(dt[,c(\"",x,"\",\"",y,"\"),with=F],\"",y,"\",breaks_list = list(",x,"=c(",breaks,")),print_step=0L)")))
+  show_binadj_plot = function(dt, y, x, breaks, stop_limit) {
+    text_woebin = paste0("bin_adj=woebin(dt[,c(\"",x,"\",\"",y,"\"),with=F], \"",y,"\", breaks_list=list(",x,"=c(",breaks,")), ", ifelse(stop_limit=="N","stop_limit = \"N\", ",NULL), "print_step=0L)")
+
+    eval(parse(text = text_woebin))
+
 
     ## print adjust breaks
     breaks_bin = setdiff(sub("^\\[(.*),.+", "\\1", bin_adj[[1]]$bin), c("-Inf","Inf","missing"))
@@ -768,21 +776,42 @@ woebin_adj = function(bins, dt, y) {
     print(woebin_plot(bin_adj)[[1]])
 
     # # breaks
-    if (breaks == "") breaks = breaks_bin
+    if (breaks == "" || is.null(breaks)) breaks = breaks_bin
 
     return(breaks)
   }
 
+  # class of variables
+  vars_class = dt[,sapply(.SD, class), .SDcols = bins[,unique(variable)]]
+  vars_class = data.table(variable = names(vars_class), x_class = vars_class)
+  # breakslist of bins
+  bins_breakslist = bins[
+    , bin2 := sub("^\\[(.*),.+", "\\1", bin)
+    ][!(bin2 %in% c("-Inf","Inf","missing"))
+    ][vars_class, on="variable"
+    ][, .(x_breaks = paste(ifelse(x_class == "numeric", bin2, paste0("\"", bin2, "\"")), collapse = ", ")), by=variable]
+
   # x variables
-  xs = bins[,unique(variable)]
+  if (all_var == FALSE) {
+    xs = union(
+      bins[count_distr < count_distr_limit, unique(variable)],
+      bins[bin != "missing"][, badprob2 := badprob >= shift(badprob, type = "lag"), by="variable"][!is.na(badprob2), length(unique(badprob2)), by="variable"][V1 > 1, variable]
+    )
+  } else {
+    xs = bins[,unique(variable)]
+  }
+
   xs_len = length(xs)
-  break_list = NULL
-  for (i in 1:xs_len) {
+  i = 1
+  breaks_list = NULL
+  while (i <= xs_len) {
+    # gn = readline(paste0("> Go to nth Var (e.g.: g4,Go 4th): "))
+
     # x variable
     x = xs[i]
-    cat("--------", paste0(i,"/",xs_len), x, "--------\n")
+    cat("--------", paste0(i, "/", xs_len), x, "--------\n")
     bin = bins[variable==x]
-    breaks = NULL
+    breaks = stop_limit = NULL
 
     # basic information of x variable ------
     ## class
@@ -803,11 +832,8 @@ woebin_adj = function(bins, dt, y) {
       }
     }
     ## current breaks
-    breaks_bin = setdiff(sub("^\\[(.*),.+", "\\1", bin$bin), c("-Inf","Inf","missing"))
-    breaks_bin = ifelse(
-      is.numeric(dt[[x]]),
-      paste0(breaks_bin, collapse=", "),
-      paste0(paste0("\"",breaks_bin,"\""), collapse=", "))
+    breaks_bin = bins_breakslist[variable == x, x_breaks]
+
     cat("> Current breaks: ","\n", breaks_bin,"\n","\n")
     ## woebin plotting
     plist = woebin_plot(bin)
@@ -817,20 +843,26 @@ woebin_adj = function(bins, dt, y) {
     while (menu(c("No", "Yes"), title=paste0("> Adjust breaks for (", i, "/", xs_len, ") ", x, "?")) == 2) {
       breaks = readline("> Enter modified breaks: ")
       breaks = gsub("^[,\\.]+|[,\\.]+$", "", breaks)
+      if (breaks == "N") {
+        stop_limit = "N"
+        breaks = NULL
+      } else {
+        stop_limit = NULL
+      }
 
-      tryCatch(show_binadj_plot(dt, y, x, breaks), finally=next)
+      tryCatch(breaks <- show_binadj_plot(dt, y, x, breaks, stop_limit), finally=next)
     }
 
 
-    if (is.null(breaks) || breaks == "") breaks = breaks_bin
-    # break_list
-    if (length(breaks)>0) {
-      break_list = c(break_list, paste0(x,"=c(",breaks,")"))
-      break_list = paste0(break_list, collapse=", \n")
-    }
+    if (!(is.null(breaks) || breaks == "")) bins_breakslist[variable == x][["x_breaks"]]  <- breaks
+    i = i + 1
   }
 
-  break_list = paste0(c("list(",break_list,")"),collapse = "\n")
-  cat(break_list,"\n")
-  return(break_list)
+
+  breaks_list = paste0(bins_breakslist[, paste0(variable, "=c(", x_breaks, ")")], collapse = ", \n ")
+  breaks_list = paste0(c("list(", breaks_list, ")"), collapse = "\n ")
+
+
+  cat(breaks_list,"\n")
+  return(breaks_list)
 }
